import add_close_braces
import process_line
import join_lines
import finish_semicolons

def _(src):
    raw_lines = src.split("\n")[0:-1]
    indent = 0

    lines = [[0,'','-']]
    for line in raw_lines:
        line = line.split('    ')
        indent = process_line._(lines, line, indent)
    add_close_braces._(lines, indent, 0)

    return join_lines._(finish_semicolons._(lines[1:]))



import os
import read_class_files as read
import process_class_files as process

def _(path, namespace_depth):
    top = {'namespaces': {}, 'classes': {}}

    for root, dirs, files in os.walk(path):
        if len(files) == 0:
            continue
        address = root.split(os.sep)
        class_name = address.pop()

        parent = top
        for namespace in address:
            if (namespace not in parent['namespaces']):
                parent['namespaces'][namespace] = {
                        'namespaces': {}, 'classes': {}}
            parent = parent['namespaces'][namespace]

        methods = read._(address, class_name, files)
        methods = process._(methods, address[namespace_depth:], class_name)
        parent['classes'][class_name] = methods
    return top



import construct_pre_method_string

def _(methods, address, class_name):
    ret = construct_pre_method_string._(methods, address, class_name)

    string_methods = []
    for name, body in methods.items():
        string_method = ''
        if 'test' in class_name:
            string_method += "\n        [Test]\n        "
        string_method += '\n        '.join(body)
        string_methods.append(string_method)

    ret += '\n        ' + '\n\n        '.join(string_methods) + '\n    }\n}'
    return ret[1:]



def _(lines, start, end):
    line_index = len(lines) - 1
    while(lines[line_index][2] == '-' and line_index > 0):
        line_index -= 1
    line_index += 1

    while (start != end):
        start -= 1
        lines.insert(line_index, [start, '}', ''])
        line_index += 1



def _(methods, address, class_name):
    ret = []

    if 'using.cs' in methods:
        for line in methods['using.cs']:
            ret += [line]
        ret += ['']
        del(methods['using.cs'])

    ret += ['namespace ' + '.'.join(address)]
    ret += ['{']
    if 'test' in class_name:
        ret += ["    using NUnit.Framework;"]
        ret += [""]
        ret += ["    [TestFixture]"]
    ret += ["    [System.Serializable]"]


    if '_.cs' in methods:
        for line in methods['_.cs']:
            ret += ['    ' + line]
        del(methods['_.cs'])
    else:
        ret += ["    public class " + class_name]

    if '}' not in ret[-1]:
        ret += ["    {"]
        ret += ["    }"]

    ret = '\n' + '\n'.join(ret[:-1])
    return ret



def _(line):
    string = ''.join(line).strip()
    if string[0:4] == 'for ':
        return False
    if string[0:2] == 'if':
        return False
    if 'public class' in string:
        return False
    if 'public' in string and ')' in string:
        return False
    if string == '':
        return False
    if string[0] == '[':
        return False
    if string[0] == '(':
        return False
    if string[0:2] == '//':
        return False
    if string == 'get':
        return False
    # filter for more open than closed brackets
    return True



import is_complete_line
import add_close_braces
import get_new_indent

def _(lines, line, indent):
    new_indent = get_new_indent._(line)
    if new_indent > indent + 1:
        lines.append([new_indent, line, '>'])
        return indent

    semicolon = ';' if is_complete_line._(line) else ''

    if new_indent == indent + 1:
        lines.append([indent, '{', ''])
        lines.append([new_indent, line, semicolon])
        return new_indent

    add_close_braces._(lines, indent, new_indent)
    indent = new_indent

    lines.append([indent, line, semicolon])

    return indent



import os
import process_method
import replace_keywords

def _(address, class_name, files):
    methods = {}
    for filename in files:
        if '.cs' not in filename:
            continue
        path = os.sep.join(address) + '/' + class_name + '/' + filename
        with open(path, 'r') as f:
            method = process_method._(f.read())
            methods[filename] = replace_keywords._(method, filename, class_name)
    return methods



def _(method, filename, classname):
    methodname = filename.split('.')[0]
    def f(i):
        i = i.replace('_method', methodname)
        return i.replace('_class', classname)
    return map(f, method)




import os

def _(src, path, namespace_depth):
    for name, namespace in src['namespaces'].items():
        if namespace_depth == 0:
            _(namespace, path + os.sep + name, 0)
        else:
            _(namespace, path, namespace_depth - 1)

    for name, body in src['classes'].items():
        if not os.path.exists(path):
            os.makedirs(path)

        with open(path + os.sep + name + '.cs', 'w') as f:
            f.write(body)



def _(lines):
    ret = []
    switching = False
    for line in reversed(lines):
        if line[2] == '>':
            if not switching:
                switching = True
                line[2] = ';'
            else:
                line[2] = ''
        elif line[2] == ';' and switching:
            line[2] = ''
            switching = False
        ret.insert(0,line)
    return ret



def _(lines):
    ret = []
    for line in lines:
        string = ''
        for i in range(line[0]):
            string += '    '
        if line[2] == '-':
            line[2] = ''
        string += ''.join(line[1]) + line[2]
        ret.append(string)
    return ret



import collate_cs
import sys

if len(sys.argv) < 4:
    print('usage:')
    print('collate before_path after_path depth')
    exit()

collate_cs._(sys.argv[1], sys.argv[2], int(sys.argv[3]))


__version__ = "0.21"

__title__ = "collate_cs"
__description__ = "thin -> cs"
__uri__ = "https://bitbucket.org/Cogbot/collate_cs"

__author__ = "Luke Aver"
__email__ = "luke.avery@live.co.uk"

__license__ = "MIT"
__copyright__ = "Copyright (c) 2017 Luke Avery"



import collate_cs
import sys

def main():
    print('running')
    if len(sys.argv) < 4:
        print('usage:')
        print('collate before_path after_path depth')
        exit()

    collate_cs._(sys.argv[1], sys.argv[2], int(sys.argv[3]))

if __name__ == "__main__":
    main()
